import Destroyable from "./Destroyable";
/**
 * Cancellation token is an object that provides a signal on destruction for the bound asynchronous operations to
 * stop working. Examples of asynchronous operations that support cancellation are `defer` and `request`. In comparison
 * to "destroyable promise" approach, cancellation tokens are compatible to async/await syntax.
 */
export default class CancelToken implements Destroyable {
    private _dispatcher;
    /**
     * Indicates if the token is already canceled, i.e. destroyed.
     */
    get cancelled(): boolean;
    /**
     * Adds a handler function to call on taken cancellation. If the token is already canceled, calls the
     * function immediately and returns `dummyDestroyable`. Else registers the callback for a one-time call on
     * token destruction and returns the attachment. Destroying the returned attachment results in handler
     * function removal.
     * @param handler Token cancelation callback.
     * @param scope Handler call scope.
     * @returns Handler attachment.
     */
    addHandler(handler: () => any, scope?: any): Destroyable;
    /**
     * Cancels the token. Calls all registered handler functions. An attempt to add more handler functions
     * after the token destruction results in their immediate calling.
     */
    destroy(): void;
}
/**
 * Helper function that wraps an abstract asynchronous operation with a promise supporting cancellation tokens.
 * @param run Asyncronous operation initiation callback.
 * @param cancel Asyncronous operation cancelation callback.
 * @param cancelToken Optional cancelation token.
 * @returns Promise representing the operation with cancelation token support.
 */
export declare function runAsync<T>(run: (resolve: (value?: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void, cancel: () => void, cancelToken?: CancelToken): Promise<T>;
