import Destroyable from './Destroyable';
import IClass from './IClass';
/**
 * Introduces object aggregation support.
 * If you call `a.own(b)`, then **b** is destroyed automatically on **a** destruction.
 * You can aggregate any object implementing `Destroyable`.
 *
 * See online documentation for details.
 */
declare class Class implements IClass {
    private _ownagePool;
    /**
     * Aggregates the object. It means that the specified object is automatically destroyed
     * on this object destruction. The aggregated objects are destroyed in reverse order.
     * Returns the aggregated object, which makes it easy to use in field definition.
     *
     * @param obj Object to aggregate.
     */
    own<T extends Destroyable>(obj: T): T;
    /**
     * Aggregates the object. It means that the specified object is automatically destroyed
     * on this object destruction. The aggregated objects are destroyed in reverse order.
     * Returns this object, which makes it easy to use in object instantiation.
     *
     * @param obj Object to aggregate.
     */
    owning(obj: Destroyable): this;
    /**
     * Class destructor invocation method. Destroys all aggregated objects and calls `destroyObject` method.
     * You must call this method explicitly from outside, because JavaScript doesn't support automatic class destructor
     * calling.
     */
    destroy(): void;
    /**
     * Class destructor implementation. Called inside `destroy` method *after aggregated objects destruction*.
     * The logic of class instance destruction should be implemented here. If you override this method,
     * remember to call `super.destroyObject()` at the end of the method.
     */
    protected destroyObject(): void;
}
export default Class;
