/// <reference types="jquery" />
import AbstractTemplate from './AbstractTemplate';
import Bindable from './Bindable';
import Class from './Class';
import ComponentArray from './component/ComponentArray';
import ComponentBindable from './component/ComponentBindable';
import ComponentSet from './component/ComponentSet';
import Destroyable from "./Destroyable";
import IBindableMap from "./IBindableMap";
import ReadonlyBindableArray from './ReadonlyBindableArray';
import ReadonlyBindableSet from './ReadonlyBindableSet';
import TemplateOutput from './TemplateOutput';
/**
 * Base class of UI component.
 *
 * Features:
 *
 * - Rendering by HTML template.
 * - Direct access to component elements.
 * - jQuery interface for element manipulations.
 * - Convenient API for data binding and child component management.
 */
export default class Component extends Class {
    private _parent;
    private _el;
    private _children;
    private _wasAfterAppend;
    private _template;
    private __elements;
    private __bindables;
    private __arrays;
    private __sets;
    /**
     * Plain objects of this class can be constructed. They can be used as dummy components or simple containers.
     */
    constructor();
    /**
     * Parent component. The property is available from component rendering beginning.
     */
    get parent(): Component;
    /**
     * Root element. Field is available from component rendering beginning.
     */
    get el(): JQuery<HTMLElement>;
    /**
     * Mutable named child components. Use this map to add child components in place of
     * elements with corresponding `jwid`. Field is available from component rendering beginning.
     */
    get children(): IBindableMap<string, Component>;
    /**
     * Component template.
     */
    get template(): AbstractTemplate;
    get _elements(): Map<string, JQuery<HTMLElement>>;
    get _bindables(): Map<string, ComponentBindable>;
    get _arrays(): Set<ComponentArray>;
    get _sets(): Set<ComponentSet>;
    destroy(): void;
    /**
     * Component life stage method. Called during component rendering after HTML template parsing and initialization
     * of references to all elements of the template. Called before `render<ChildId>` methods and
     * `afterRender` method. It is convenient to perform some preliminary action here before child
     * components creation. But you are already able to create child components here.
     * Call `super.beforeRender()` at the first line of the method.
     */
    protected beforeRender(): void;
    /**
     * Component life stage method. Called after `beforeRender` method and `render<ChildId>` methods.
     * You should assign all elements' attributes here, create child components,
     * bind event handlers and fill component with interactivity.
     * Call `super.afterRender()` at the first line of the method.
     */
    protected afterRender(): void;
    /**
     * Component life stage method. Called after first-time component appearing in HTML DOM and UI components tree.
     * Component layouting should be performed here (element size computing).
     * Component rendering is finished here.
     * Call `super.afterAppend()` at the first line of the method.
     */
    protected afterAppend(): void;
    /**
     * Component life stage method. Called during component destruction before `unrender` method call.
     * Everything that was performed in `afterAppend` method should be reverted here.
     * Call `super.releaseDom()` at the last line of the method.
     */
    protected releaseDom(): void;
    /**
     * Component life stage method. Called during component destruction before `afterDestroy` method call.
     * Everything that was performed during component
     * rendering should be reverted here. All child components are already removed by the component
     * before this method call, but the components themselves are not destroyed. You must destroy them explicitly.
     * Call `super.unrender()` at the last line of the method.
     */
    protected unrender(): void;
    /**
     * Component life stage method. Called during component destruction after `unrender` method call.
     * Everything that was performed during component construction should be reverted here.
     * Call `super.afterDestroy()` at the last line of the method.
     */
    protected afterDestroy(): void;
    /**
     * Virtual method to render the component document fragment.
     * By default, renders by template.
     */
    protected createElement(): TemplateOutput;
    /**
     * Selects component rendering strategy. This method is needed only in very rare cases.
     * By default, component is rendered outside of DOM based on `main` HTML
     * template specified by `template` annotation. You can change this by `using` method call.
     */
    using(value: string | AbstractTemplate | HTMLElement | JQuery): this;
    /**
     * Renders the component. Call this method to initialize references to all elements of component and create
     * child components.
     */
    render(): this;
    /**
     * Renders component into an element. Use it to render root component only: its children must be rendered
     * using `children`, `addArray`, `addSet`, `addBindable` members.
     *
     * @param el Element to render component into.
     */
    renderTo(el: string | HTMLElement | JQuery): this;
    /**
     * Render component in place of an element. Use it to render root component only: its children must be rendered
     * using `children`, `addArray`, `addSet`, `addBindable` members.
     *
     * @param el Element to render component in place of.
     */
    renderAs(el: string | HTMLElement | JQuery): this;
    /**
     * Remove the component from DOM. Can be used for root component only (which was added via `renderTo` or `renderAs`
     * method). All child components should be removed using `children` map or owning object deletion.
     */
    remove(): this;
    /**
     * Get element by its `jwid`.
     * @param id `jwid` of the element.
     */
    getElement(id: string): JQuery<HTMLElement>;
    /**
     * Remove element by `jwid`. Element gets removed from DOM and destroyed. It is then impossible to get it by `getElement` method.
     * @param id `jwid` of the element.
     */
    removeElement(id: string): this;
    /**
     * Adds child component and synchronizes the component with the bindable.
     *
     * @param component Child component bindable.
     * @param id `jwid` of element to replace.
     */
    addBindable(component: Bindable<Component>, id: string): Destroyable;
    /**
     * Adds an array of child components and synchronizes the component with it. As opposed to `addSet` method,
     * keeps component order.
     *
     * @param source Child component array.
     * @param el `jwid` of element to add child components into. Defaults to root element (`el`) of component.
     */
    addArray(source: ReadonlyBindableArray<Component>, el?: string | HTMLElement | JQuery): Destroyable;
    /**
     * Add child component set into an element. As opposed to `addArray` method, ignores
     * component order.
     *
     * @param source Child component set.
     * @param el `jwid` of element to add child components into. Defaults to root element (`el`) of component.
     */
    addSet(source: ReadonlyBindableSet<Component>, el?: string | HTMLElement | JQuery): Destroyable;
    _afterAppend(): void;
    _initChild(component: Component): void;
    _doneChild(component: Component): void;
    _getContainerElement(el?: any): JQuery<any>;
}
