import Destroyable from "./Destroyable";
import IDispatcher from "./IDispatcher";
import Listener from "./Listener";
/**
 * Real implementation of `IDispatcher` interface that calls handler functions on `dispatch` method call (as opposed to
 * `dummyDispatcher` which doesn't).
 * Used to dispatch messages (for example, field value change) to other objects (listeners).
 */
export default class Dispatcher<M> implements IDispatcher<M> {
    private _listeners;
    /**
     * Checks if this dispatcher is dummy, i.e. it never dispatches any messages.
     * This knowledge may help you do certain code optimizations.
     */
    get dummy(): boolean;
    /**
     * Unregisters all listeners.
     */
    purge(): void;
    /**
     * Registers a listener for this dispatcher.
     *
     * Whenever a message is dispatched with `dispatch` method, the specified handler function is called.
     *
     * You can unregister the listener destroying the returned object.
     *
     * @param handler Handler function.
     * @param scope `handler` call scope.
     */
    listen(handler: (message: M) => void, scope?: any): Destroyable;
    /**
     * Dispatches a message, i.e. calls handler functions of all listeners.
     * @param message Message to dispatch.
     */
    dispatch(message?: M): void;
    _unbind(listener: Listener<M>): void;
    /**
     * If `dummy` argument is false, returns a new instance of `Dispatcher`. Else returns `dummyDispatcher`.
     * @param dummy Determines if dummy dispatcher should be used.
     */
    static make<M>(dummy: boolean): IDispatcher<M>;
}
