import Bindable from './Bindable';
import Class from './Class';
import DestroyableBindable from './DestroyableBindable';
import IProperty from './IProperty';
import Listenable from './Listenable';
/**
 * Listens source properties modification and recreates target value via mapping function.
 *
 * @param T Target property value type.
 */
declare class Mapper<T> extends Class {
    readonly sources: Bindable<any>[];
    private _create;
    private _destroy;
    private _scope;
    private _sourceValues;
    private _targetValue;
    private _target;
    private _viaNull;
    /**
     * @param sources Source properties.
     * @param create Mapping function.
     * @param config Configuration.
     */
    constructor(sources: Bindable<any>[], create: Mapper.CreateCallback<T>, config?: Mapper.FullConfig<T>);
    /**
     * Target property.
     */
    get target(): Bindable<T>;
    /**
     * @inheritDoc
     */
    protected destroyObject(): void;
    /**
     * Listens to the specified dispatcher and issues target value recalculation on any message.
     * @param dispatcher Dispatcher.
     * @returns this
     */
    listen(dispatcher: Listenable<unknown>): this;
    /**
     * Watches over the specified property and issues target value recalculation on its change.
     * @param property Property.
     * @returns this
     */
    bind(property: Bindable<unknown>): this;
    /**
     * Updates target property forcibly.
     */
    update(): void;
    private _done;
}
declare namespace Mapper {
    /**
     * Mapper's `create` callback signature.
     *
     * @param T Target property value type.
     */
    interface CreateCallback<T> {
        (...sourceValues: any[]): T;
    }
    /**
     * Mapper's `destroy` callback signature.
     *
     * @param T Target property value type.
     */
    interface DestroyCallback<T> {
        (targetValue: T, ...sourceValues: any[]): any;
    }
    /**
     * Configuration of `mapProperties` function. Partial configuration of Mapper.
     */
    interface Config<T> {
        /**
         * Destroys target property value if specified.
         */
        readonly destroy?: DestroyCallback<T>;
        /**
         * `create` and `destroy` call scope.
         * Defaults to mapper itself.
         */
        readonly scope?: any;
        /**
         * Reverses mapper updating flow.
         */
        readonly viaNull?: boolean;
    }
    /**
     * Full configuration of Mapper.
     *
     * @param T Target property value type.
     */
    interface FullConfig<T> extends Config<T> {
        /**
         * Target property. By default, created automatically.
         */
        readonly target?: IProperty<T>;
    }
    /**
     * Mapper by reducer. Kind of mapper optimized for working with collections of similar properties.
     */
    class ByReducer<T, U> extends Class {
        readonly sources: Bindable<T>[];
        private callback;
        private initial;
        private _target;
        /**
         * @param sources Source bindables.
         * @param reducer Mapping reducer.
         * @param target Target property.
         */
        constructor(sources: Bindable<T>[], callback: (accumulator: U, item: T) => U, initial: U, target?: IProperty<U>);
        /**
         * Target property.
         */
        get target(): Bindable<U>;
        private _update;
        private _bind;
    }
}
export default Mapper;
/**
 * @param sources Source properties.
 * @param create Mapping function.
 * @param config Configuration.
 * @returns Target property.
 */
export declare function mapProperties<T>(sources: Bindable<any>[], create: Mapper.CreateCallback<T>, config?: Mapper.Config<T>): DestroyableBindable<T>;
export declare function mapPropertiesByReducer<T, U>(sources: Bindable<T>[], callback: (accumulator: U, item: T) => U, initial: U): DestroyableBindable<U>;
