import Bindable from "./Bindable";
import Class from "./Class";
import DestroyableBindable from "./DestroyableBindable";
import IDispatcher from "./IDispatcher";
import IProperty from "./IProperty";
import Listenable from "./Listenable";
import { default as Mapper } from "./Mapper";
/**
 * Container for a value. Provides basic data binding functionality.
 */
export default class Property<V> extends Class implements IProperty<V> {
    protected value: V;
    private _ownsValue;
    protected _onChange: IDispatcher<Bindable.ChangeMessage<V>>;
    /**
     * Constructs a property and sets initial value.
     * @param value Initial value.
     * @param silent If true, uses `dummyDispatcher` implementation for `onChange`.
     */
    constructor(value?: V, silent?: boolean);
    protected destroyObject(): void;
    /**
     * Checks if this property never dispatches any messages. This knowledge may help you do certain code optimizations.
     */
    get silent(): boolean;
    /**
     * Property value is changed. Triggered in result of `set` method call if the value has been changed.
     */
    get onChange(): Listenable<Bindable.ChangeMessage<V>>;
    /**
     * Returns current property value.
     * Think twice before calling this method - probably it makes sense to use some kind of binding instead?
     */
    get(): V;
    /**
     * Changes the property value and dispatches a change message if the value has truly been changed.
     * @param value New value to set.
     */
    set(value: V): void;
    /**
     * Makes this property an owner of its value. It means that the value is
     * destroyed automatically on reassignment or destruction of the
     * property.
     */
    ownValue(): this;
    /**
     * Builds a new property containing the result of the callback function called
     * on this property value. To stop synchronization, destroy the result property.
     * In comparison to `mapDestroyable` method, doesn't destroy the previously assigned target values.
     * To map multiple properties, use `Mapper`.
     *
     * @param create Mapping function.
     * @param config Mapping configuration.
     */
    map<U>(create: (value: V) => U, config?: Mapper.Config<U>): DestroyableBindable<U>;
}
