import Bindable from "./Bindable";
import IBindableSet from './IBindableSet';
import Listenable from './Listenable';
/**
 * Bindable readonly wrapper over a native set.
 */
interface ReadonlyBindableSet<T> extends Iterable<T> {
    /**
     * Iterates over values in the set.
     */
    [Symbol.iterator](): IterableIterator<T>;
    /**
     * The set never dispatches any messages. This knowledge may help you do certain code optimizations.
     */
    readonly silent: boolean;
    /**
     * Property containing number of values in the set.
     */
    readonly size: Bindable<number>;
    /**
     * Internal representation of the set.
     */
    readonly native: ReadonlySet<T>;
    /**
     * Values are removed from the set and/or values are added to the set.
     */
    readonly onSplice: Listenable<IBindableSet.SpliceResult<T>>;
    /**
     * The set is cleared. Passes old contents as a message.
     */
    readonly onClear: Listenable<ReadonlySet<T>>;
    /**
     * The set is changed. Dispatched right after any another message.
     */
    readonly onChange: Listenable<void>;
    /**
     * Checks if an value is present in the set.
     */
    has(value: T): boolean;
    /**
     * Iterates through the set values. Calls the specified function for all values.
     * @param callback Callback function.
     */
    forEach(callback: (value: T) => void): void;
    /**
     * Detects `splice` method arguments to adjust the set contents to `newContents`.
     * Determines value bunches to be removed and added.
     * @param newContents New set contents.
     * @returns `splice` method arguments. If no method call required, returns undefined.
     */
    detectSplice(newContents: Iterable<T>): IBindableSet.SpliceParams<T>;
}
export default ReadonlyBindableSet;
