import Bindable from './Bindable';
import Class from './Class';
import Destroyable from './Destroyable';
import IProperty from './IProperty';
import Router from "./Router";
/**
 * Creates a router that manages two mapping of properties:
 *
 * * `paths` which exposes string path properties for child routers if neccessary;
 * * `expanded` which exposes boolean "expanded" properties for child UI panels.
 *
 * This allows you to render your content as a fixed array of panels representing the concurrent routes.
 */
export default class RoutingNode extends Class {
    private _initialized;
    private _updating;
    /**
     * Provides paths to bind child routers to, by name. Only one route is active at a time, but their paths
     * always exist regardless of their activity.
     */
    readonly paths: ReadonlyMap<string, IProperty<string>>;
    /**
     * Provides "expanded" flags to bind child panels to, by name. Support two-way binding.
     */
    readonly expanded: ReadonlyMap<string, IProperty<boolean>>;
    /**
     * Default route this node was initialized with.
     */
    readonly defaultRoute: string;
    /**
     * Router that manages this node. Node creates this router automatically. You should pass this router to
     * child components as their parent router for further routing.
     */
    readonly router: Router<Destroyable>;
    /**
     * Creates router node, assigns its properties to initial values and starts synchronization.
     * @param config Node configuration.
     */
    constructor(config: RoutingNode.Config);
}
export declare namespace RoutingNode {
    /**
     * Router.Node configuration.
     */
    interface Config {
        /**
         * Router name.
         */
        readonly name?: string;
        /**
         * Parent router.
         */
        readonly parent?: Router<any>;
        /**
         * Path to bind the router to.
         */
        readonly path?: Bindable<string>;
        /**
         * Fixed array of routes to manage by this node. For every name in this list, corresponding properties will be
         * created in `paths` and `expanded` dictionaries of the node.
         */
        readonly routes: Iterable<string>;
        /**
         * Initial "expanded" status of routes or initial routes to expand. Defaults to false (all routes are
         * collapsed).
         */
        readonly expanded?: boolean | Iterable<string>;
        /**
         * Default route. If the initial path is blank (""), the router performs a redirection to this route, i.e.
         * expands one of the panels. Doesn't work after initialization.
         */
        readonly defaultRoute?: string;
    }
}
